C***********************************************************************
C*                     D U B N A                                       *
C*          JOINT INSTITUTE FOR NUCLEAR RESEARCH                       *
C*         LABORATORY OF INFORMATION TECHNOLOGIES                      *
C*             PUZYNIN I.V.,PUZYNINA T.P.                              *
C*                   SLIP1.FOR                                         *
C*  KFKI-74-34 HUNGARIAN ACADEMY OF SCIENCES, BUDAPEST, 1974, P.93-107 *
C*   THIS PROGRAM COMPLEX IS INTENDING FOR SOLVING OF STURM-           *
C*         LIOUVILLE PROBLEM FOR DIFFERENTIAL EQUATION                 *
C*          D2Y/DX2+2*P(X)*DY/DX+(Q(X)-EV*R(X))*Y=0                    *
C*             WITH  BOUNDARY  CONDITIONS                              *
C*               D1*DY/DX+F1*Y=0  AT X=A                               *
C*               D2*DY/DX+F2*Y=0  AT X=B                               *
C*       USING CONTINUE ANALOG OF NEWTON'S METHOD                      *
C*       DISCRETE SCHEMES WITH PRECISION O(H**2)                       *
C*                                                                     *
C* TEST: MORZE POTENTIAL                                               *
C*   NUMBER OF ZEROS OF EIGEN FUNCTION Y(I)  NZERO=0                   *
C*   P(X)=0, Q(X)=-2MD( EXP(-2A1(X-X0)) - 2EXP(-A1(X-X0)) ), R(X)=1    *
C*   BOUNDARY CONDITIONS D1=1, F1=SQRT(EV)-SQRT(2*AM*D)*EXP(AP*X0)     *
C*   BOUNDARY CONDITIONS D2=1, F2=SQRT(EV)-SQRT(2*AM*D)*EXP(-AP*(B-X0))*
C*   A=0.  B=20. H=20/1600                                             *
C***********************************************************************

      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(NDIM=2001)
      DIMENSION P(NDIM),Q(NDIM),R(NDIM),Y0(NDIM),Y(NDIM),X(NDIM)
      COMMON /AMDX/ AM,D,X0,AP
      COMMON /ALPH/ ALPHA
C*  FILE SLIP1.DAT FOR OUTPUT.DAT
      OPEN(1,FILE='SLIP1.DAT',STATUS='UNKNOWN')
      WRITE(1,1000)
      WRITE(1,1001)

C********* PHYSICAL VALUES
      AM1=1836.152D0
      AM2=1836.152D0
      AM3=206.769D0
      AP=0.67D0
      D=0.1055D0
      X0=2.15D0
      NZERO=0
C      AM=AM1*AM2*(AM1+AM2+AM3)/(AM3*(AM1+AM2)**2)
      AM=4.69D0
      ALPHA=(AM1+AM2+AM3)/(AM3*(AM1+AM2)*27.211652D0)

C********* PARAMETERS  FOR DISCRETE SCHEME
      A=0.D0
      B=20.D0
      N=1601
      H=(B-A)/(N-1)

      PRINT 1000
      PRINT 1001
      PRINT 1002
      PRINT 1003,AM1,AM2,AM3,AP,D,X0
      WRITE(1,1003)AM1,AM2,AM3,AP,D,X0
      PRINT 1004,AM,ALPHA,NZERO
      WRITE(1,1004)AM,ALPHA,NZERO
      PRINT 1011,A,B,N,H
      WRITE(1,1011)A,B,N,H

      DO 1 I=1,N
      X(I)=A+H*(I-1)
      P(I)=0.D0
      R(I)=1.D0
      Q(I)=2.D0*AM*D*(DEXP(-2.D0*AP*(X(I)-X0))-2.D0*DEXP(-AP*(X(I)-X0)))
      Q(I)=-Q(I)
    1 CONTINUE

      PRINT 1005
      PRINT 1006
      NPR=N/4
      NPR=1
      PRINT 1007,(I,X(I),P(I),Q(I),R(I),I=1,N,NPR)

C********* PARAMETERS  FOR SLIP1
      EV0=0.6D0
      T0=.01D0
      EPS=1.D-4

      PRINT 1008,EV0,T0,EPS

C********* POSSIBILITY  FOR INPUT OF NEW VALUES: EV0,T0,EPS
C      WRITE (*,*) 'INPUT NEW:  EV0 T0  EPS '
C      READ (*,*) EV0,T0,EPS
C      PRINT 1008,EV0,T0,EPS

      WRITE(1,1008)EV0,T0,EPS

      CALL SLIV0(A,B,N,EV0,NZERO,H,Y0)

C********* PARAMETERS  FOR SLIP1
      NT=200
      LST=3
      LPR=10
      LX=20

      CALL SLIP1(A,B,N,P,Q,R,EV0,Y0,EV,Y,EPS,NT,LST,LPR,LX,T0,X)

C******** CALCULATION OF ANALYTICAL SOLUTION (EV0,Y0)
      CALL SLIV(A,B,N,EV0,Y0,NZERO)

      WRITE(1,1009)EV0,EV
      WRITE(1,*)
      WRITE(1,1013)
      NWR=8
      WRITE(1,1010)(J,X(J),Y0(J),Y(J),J=1,N,NWR)
      WRITE(1,1012)
      PRINT 1013
      PRINT 1010,(J,X(J),Y0(J),Y(J),J=1,N,NWR)
      PRINT 1009,EV0,EV
      PRINT 1012

 1000 FORMAT(10X,' SLIP1.FOR  WITH TEST: MORZE POTENTIAL')
 1001 FORMAT(1X,'TEST: SOLUTION (EV,Y(I)) FOR NUMBER OF ZEROS NZERO=0')
 1002 FORMAT(//,5X,40(1H*),/,13X,'PHYSICAL VALUES ')
 1003 FORMAT(1X,'M1=',F9.3,2X,'M2=',F9.3,2X,'M3=',F8.3,2X,'A1=',F5.3,
     ,2X,'D=',F7.4,2X,'X0=',F5.2)
 1004 FORMAT(1X,'M=',D16.9,5X,'ALPHA=',D16.9,5X,'NUMBER OF ZEROS',I4)
 1005 FORMAT(5X,'THE TABLE OF THE MORSE POTENTIAL')
 1006 FORMAT(5X,'I',5X,'X',5X,'P',12X,'Q',12X,'R')
 1007 FORMAT(1X,I4,F10.4,F5.2,D19.8,F6.2)
 1008 FORMAT(5X,'EV0=',D16.9,' T0=',F8.5,' EPS=',F9.7)
 1009 FORMAT(5X,' EV_ANALYT=',D16.9,' EV=',D16.9)
 1010 FORMAT(1X,I10,F14.6,2D17.8)
 1011 FORMAT(10X,'A=',F5.2,' B=',F5.2,' N=',I4,' H=',F8.6)
 1012 FORMAT(8X,'****************END OF TEST**********************')
 1013 FORMAT('         I        X(I)        Y_ANALYT(I)       Y(I)' )

      END
C**********************************************************************
      SUBROUTINE SLIV0(A,B,N,EV0,NZERO,H,Y0)
C* INITIAL APPROXIMATION Y0(I)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y0(N)
      XAS=4.D0
      IF(NZERO.EQ.1)XAS=9.D0
      P=3.14159265D0*(NZERO+1.D0)/XAS
      C=XAS*DSIN(P*XAS)
      DO 1 I=1,N
      X=A+(I-1)*H
      Y0(I)=X*DSIN(P*X)
      IF(X.GT.XAS)Y0(I)=C*DEXP(-DSQRT(EV0)*X)
  1   CONTINUE
      RETURN
      END
C**********************************************************************
      SUBROUTINE SLIP1(A,B,N,P,Q,R,EV0,Y0,EV,Y,EPS,NT,LST,LPR,LX,T0,X)
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(NDIM=2001)
      DIMENSION P(N),Q(N),R(N),Y0(N),Y(N),X(N)
      DIMENSION AN(NDIM),BN1(NDIM),BN2(NDIM),V1(NDIM),V2(NDIM)
      ONE=1.D0
      TSUM=0.D0
      NN=N-1
      NNN=N-2
      H=(B-A)/NN
      DO 1 I=1,N
      X(I)=A+H*(I-1)
    1 CONTINUE
      DELTA=0.D0
      XDEL=X(1)
      IDEL=1
      DO 3 I=2,NN
      RD=DABS ((Y0(I+1)-2.D0*Y0(I)+Y0(I-1))/H**2+P(I)*(Y0(I+1)-Y0(I-1))/
     /H+(Q(I)-EV0*R(I))*Y0(I))
      IF(DELTA-RD) 2,2,3
    2 DELTA=RD
      XDEL=X(I)
      IDEL=I
    3 CONTINUE
      PRINT 999,DELTA,IDEL,XDEL,EV0
      M=1
      TK=T0
      EV=EV0
      DO 4 K=1,NT
      CALL D1F1(EV,A,D1,DD1,F1,DF1)
      AL10=-3.D0*D1+2.D0*H*F1
      AL11=4.D0*D1
      AL12=-D1
      DY=-3.D0*Y0(1)+4.D0*Y0(2)-Y0(3)
      SS1=-AL10*Y0(1)-AL11*Y0(2)-AL12*Y0(3)
      SS2=-DD1*DY-2.D0*H*DF1*Y0(1)
      DO 5 I=2,NN
      AM1=1.D0+H*P(I)
      AM0=-2.D0+H**2*(Q(I)-EV*R(I))
      AMM1=1.D0-H*P(I)
      R1=-Y0(I+1)+2.D0*Y0(I)-Y0(I-1)-P(I)*(Y0(I+1)-Y0(I-1))*H-(Q(I)-
     -EV*R(I))*Y0(I)*H**2
      R2=H**2*R(I)*Y0(I)
      IF(I-2) 6,6,7
    6 ZNA=AM0*AL10-AMM1*AL11
      IF(DABS(ZNA)-1.0E-200) 8,8,9
    9 AN(I)=-(AM1*AL10-AMM1*AL12)/ZNA
      BN1(I)=(R1*AL10-AMM1*SS1)/ZNA
      BN2(I)=(R2*AL10-AMM1*SS2)/ZNA
      GO TO 5
    7 ZNA=AM0+AMM1*AN(I-1)
      IF(DABS(ZNA)-1.0E-200) 10,10,11
   11 AN(I)=-AM1/ZNA
      BN1(I)=(R1-AMM1*BN1(I-1))/ZNA
      BN2(I)=(R2-AMM1*BN2(I-1))/ZNA
    5 CONTINUE
      CALL D2F2(EV,B,D2,DD2,F2,DF2)
      ALN2=D2
      ALN1=-4.D0*D2
      ALN0=3.D0*D2+2.D0*H*F2
      DY=Y0(N-2)-4.D0*Y0(N-1)+3.D0*Y0(N)
      TN1=-ALN2*Y0(N-2)-ALN1*Y0(N-1)-ALN0*Y0(N)
      TN2=-DD2*DY-2.D0*H*DF2*Y0(N)
      ZNA=ALN2*AN(N-2)*AN(N-1)+ALN1*AN(N-1)+ALN0
      IF(DABS(ZNA)-1.0E-200) 12,12,13
   13 V1(N)=(TN1-ALN2*BN1(N-2)-ALN2*AN(N-2)*BN1(N-1)-ALN1*BN1(N-1))/ZNA
      V2(N)=(TN2-ALN2*BN2(N-2)-ALN2*AN(N-2)*BN2(N-1)-ALN1*BN2(N-1))/ZNA
      DO 14 I=1,NNN
      V1(N-I)=AN(N-I)*V1(N-I+1)+BN1(N-I)
      V2(N-I)=AN(N-I)*V2(N-I+1)+BN2(N-I)
   14 CONTINUE
      IF(DABS(AL10)-1.0E-200) 15,15,16
   16 V1(1)=(-AL11*V1(2)-AL12*V1(3)+SS1)/AL10
      V2(1)=(-AL11*V2(2)-AL12*V2(3)+SS2)/AL10
      SY2=Y0(1)**2+Y0(N)**2
      SYV1=Y0(1)*V1(1)+Y0(N)*V1(N)
      SYV2=Y0(1)*V2(1)+Y0(N)*V2(N)
      DO 17 I=2,NN
      SY2=SY2+2.D0*Y0(I)**2
      SYV1=SYV1+2.D0*Y0(I)*V1(I)
      SYV2=SYV2+2.D0*Y0(I)*V2(I)
   17 CONTINUE
      SY2=SY2*H/2.D0
      SYV1=SYV1*H/2.D0
      SYV2=SYV2*H/2.D0
      CALL ASIM(A,B,EV,Y0(N),SYA2,SYDYA)
      ZNA=SYV2+SYDYA
      IF(DABS(ZNA)-1.0E-200) 18,18,19
   19 AMJU=-(.5D0*(SY2+SYA2-1.D0)+SYV1)/ZNA
      DO 20 I=1,N
      Y(I)=Y0(I)+TK*(V1(I)+AMJU*V2(I))
   20 CONTINUE
      EV1=EV+TK*AMJU
      DELTA1=0.D0
      IDEL=1
      XDEL=X(1)
      DO 21 I=2,NN
      RD=DABS((Y(I+1)-2.D0*Y(I)+Y(I-1))/H**2+P(I)*(Y(I+1)-Y(I-1))/H+
     +(Q(I)-EV1*R(I))*Y(I))
      IF(DELTA1-RD) 22,22,21
   22 DELTA1=RD
      IDEL=I
      XDEL=X(I)
   21 CONTINUE
      IF(DELTA1-EPS) 23,23,24
   24 IF(DELTA1-DELTA) 25,25,26
   25 GO TO (27,28,29),LST
   27 TK=T0
      GO TO 30
   28 TK=DMIN1(ONE,2.D0*TK)
      GO TO 30
   29 TK=DMIN1(ONE,TK*(DELTA/DELTA1))
   30 DO 31 I=1,N
      Y0(I)=Y(I)
   31 CONTINUE
      DELTA=DELTA1
      TSUM=TSUM+TK
      EV=EV1
      GO TO 32
   26 GO TO (33,34,35),LST
   33 TK=T0
      GO TO 30
   34 TK=DMAX1(T0,.5D0*TK)
      GO TO 36
   35 TK=DMAX1(T0,TK*(DELTA/DELTA1))
   36 DO 37 I=1,N
      Y0(I)=Y(I)
   37 CONTINUE
      DELTA=DELTA1
      EV=EV1
      TSUM=TSUM+TK
      IF(TK-T0) 38,38,32
   38 PRINT 1000,K
      IF(TK-1.0E-200) 39,39,32
   39 PRINT 1001,K
      GO TO 42
   32 CALL CONTR(EV,EV1,EV0,RES,ISW)
      IF(K-M) 4,40,4
   40 M=M+LPR
      PRINT 1002,K,DELTA1,TK,TSUM,IDEL,XDEL
      PRINT 1003,EV1,RES
      PRINT 1004
      PRINT 1005,(I,X(I),Y(I),I=1,N,LX)

    4 CONTINUE
      PRINT 1006
      GO TO 41
   23 PRINT 1007
   41 PRINT 1002,K,DELTA1,TK,TSUM,IDEL,XDEL
      PRINT 1003,EV1,RES
      PRINT 1004
      PRINT 1005,(I,X(I),Y(I),I=1,N,LX)
      EV=EV1
   42 RETURN
    8 PRINT 1008
      PRINT 1009,K,ZNA,AM0,AL10,AMM1,AL11
      GO TO 42
   10 PRINT 1010
      PRINT 1011,K,I,ZNA,AM0,AMM1,AN(I-1)
      PRINT 1012
      PRINT 1013,(X(L),AN(L),BN1(L),BN2(L),Y0(L),L=1,I)
      GO TO 42
   12 PRINT 1014
      PRINT 1015,K,ZNA,ALN2,AN(N-2),AN(N-1),ALN1,ALN0
      GO TO 42
   15 PRINT 1016,AL10
      GO TO 42
   18 PRINT 1017,K
      PRINT 1018,ZNA,SYV2,SYDYA
      GO TO 42
  999 FORMAT(2X,'DELTA0=',D10.3,' AT X(',I4,')=',F8.4,' EV0=',
     ,D10.3)
 1000 FORMAT(5X,'VALUE OF TAU(K) IS LESS THAN TAU(0).K=',I5)
 1001 FORMAT(5X,'END OF CALCULATION.TAU(K).LE.EPS.K=',I5)
 1002 FORMAT(1X,'K=',I3,' DELTA=',D16.9,' TK=',D16.9,/,
     ,' TSUM=',D16.9,' XMAX(',I4,')=',F8.4)
 1003 FORMAT(2X,'EIGENVALUE=',D16.9,2X,'PHYSICAL VALUE=',D16.9)
 1004 FORMAT(10X,'I',10X,'X(I)',10X,'Y(I)')
 1005 FORMAT(1X,I10,F15.6,D18.9)
 1006 FORMAT(/5X,'END OF ITERATIONS.',/)
 1007 FORMAT(/5X,'SOLUTION OF EIGENVALUE PROBLEM WITH GIVEN ACCURACY')
 1008 FORMAT(5X,'LEFT BOUNDARY CONDITION IS UNSOLVABLE')
 1009 FORMAT(2X,'K=',I3,' ZNA=',D16.9,' AM0=',D16.9,' AL10=',D16.9,
     ,' AM-1=',D16.9,' AL11=',D16.9)
 1010 FORMAT(/5X,'PROGONKA IS UNCORRECT')
 1011 FORMAT(/5X,'K=',I3,' I=',I5,' ZNA=',D16.9,' AM0=',D16.9,
     ,' AM-1=',D16.9,' AN(I-1)=',D16.9)
 1012 FORMAT(/20X,'X',10X,'AN',14X,'BN1 ',13X,'BN2 ',14X,'Y(X)')
 1013 FORMAT(F23.4,4D17.9)
 1014 FORMAT(/5X,'RIGHT BOUNDARY CONDITION IS UNSOLVABLE')
 1015 FORMAT(/1X,'K=',I3,' ZN=',D12.5,' LN2=',D12.5,' AN(N-2)=',D12.5,
     ,' AN(N-1)=',D12.5,' LN1=',D12.5,' LN0=',D12.5)
 1016 FORMAT(/4X,'CONDITION IN ORIGIN IS UNCORRECT.L(1,0)=',D16.9//)
 1017 FORMAT(4X,'CALCULATION OF MJU IS UNCORRECT.K=',I3)
 1018 FORMAT(5X,'ZNA=',D16.9,' S(YV2)=',D16.9,' S(YDY)A=',D16.9)
      END
C**********************************************************************
      SUBROUTINE D1F1(EV,A,D1,DD1,F1,DF1)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /AMDX/ AM,D,X0,AP
COMMENT: FOR NZERO=0
      D1=1.D0
      DD1=0.D0
      F1=DSQRT(EV)-DSQRT(2.D0*AM*D)*DEXP(AP*X0)
      DF1=0.5D0/DSQRT(EV)
      RETURN
      END
C**********************************************************************
      SUBROUTINE D2F2(EV,B,D2,DD2,F2,DF2)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /AMDX/ AM,D,X0,AP
COMMENT: FOR NZERO=0
      D2=1.D0
      DD2=0.D0
      F2=DSQRT(EV)-DSQRT(2.D0*AM*D)*DEXP(-AP*(B-X0))
      DF2=0.5D0/DSQRT(EV)
      RETURN
      END
C**********************************************************************
      SUBROUTINE ASIM(A,B,EV,Y,SYA2,SYDYA)
      IMPLICIT REAL*8(A-H,O-Z)
      CALL YDYA(B,EV,YA,DYA)
      C=(Y/YA)**2
      P=1.D0
      ERR=0.005D0
      NTR=21
      CALL TRAP(A,B,NTR,P,EV,ERR,TYA2,TYDYA)
      SYA2=C*TYA2
      SYDYA=C*TYDYA
      RETURN
      END
C**********************************************************************
      SUBROUTINE YDYA(X,EV,YA,DYA)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /AMDX/ AM,D,X0,AP
      EVSQ=DSQRT(DABS(EV))
      S=EVSQ/AP
      DEL=DSQRT(2.D0*AM*D)/AP
      AKSI=DEL*DEXP(-AP*(X-X0))
      YA=DEXP(-AKSI)*(2.D0*AKSI)**S
      DYA=YA*(DLOG(2.D0*DEL)-AP*(X-X0))/(2.D0*AP*EVSQ)
      RETURN
      END
C**********************************************************************
      SUBROUTINE TRAP(A,B,NTR,P,EV,ERR,TYA2,TYDYA)
      IMPLICIT REAL*8(A-H,O-Z)
      N1=NTR-1
      N2=NTR-2
      H=P*(B-A)/N1
      TYA2=0.D0
      TYDYA=0.D0
      ORIG=B
    4 FIN=ORIG+P*(B-A)
      CALL YDYA(FIN,EV,YA,DYA)
      TK=YA**2
      TDK=YA*DYA
      DO 1 I=1,N2
      XI=FIN-H*I
      CALL YDYA(XI,EV,YA,DYA)
      TK=TK+2.D0*YA**2
      TDK=TDK+2.D0*YA*DYA
    1 CONTINUE
      CALL YDYA(ORIG,EV,YA,DYA)
      TK=(TK+YA**2)*H/2.D0
      TDK=(TDK+YA*DYA)*H/2.D0
      TYA2=TYA2+TK
      TYDYA=TYDYA+TDK
      DEL=DABS(TK/TYA2)
      DEL1=DABS(TDK/TYDYA)
      IF(DMAX1(DEL,DEL1)-ERR) 2,2,3
    3 ORIG=FIN
      GO TO 4
    2 RETURN
      END
C**********************************************************************
      SUBROUTINE CONTR(EV,EV1,EV0,RES,ISW)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /AMDX/ AM,D,X0,AP
      COMMON /ALPH/ ALPHA
      IF(EV1) 1,2,3
    1 PRINT 101,EV1
      PRINT 102
      EV1=DABS(EV1)
      GO TO 3
    2 PRINT 103
      PRINT 104,EV0
      EV1=EV0
    3 ISW=1
      RES=EV1/(2.D0*AM)
      RES=RES/ALPHA
      RETURN
  101 FORMAT(3X,'NEGATIVE EIGENVALUE EV1=',D16.9)
  102 FORMAT(3X,'CONTINUATION OF CALCULATION WITH EV1=ABS(EV1)')
  103 FORMAT(/3X,'EIGENVALUE VANISH.EV=EV0')
  104 FORMAT(/3X,'CONTINUATION OF CALCULATION WITH EV1=EV0=',D16.9//)
      END
C**********************************************************************
      SUBROUTINE SLIV(A,B,N,EV0,Y0,NZERO)
C  ANALYTICAL EV0, Y0(I)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y0(N)
      COMMON /AMDX/ AM,D,X0,AP
      EV0=2.D0*AM*D*(1.D0-(NZERO+0.5D0)*AP/DSQRT(2.D0*AM*D))**2
      S=DSQRT(EV0)/AP
      DELTA=DSQRT(2.D0*AM*D)/AP
      H=(B-A)/(N-1)
      IF(NZERO) 1,2,1
    2 DO 3 I=1,N
      XI=A+H*(I-1)
      AKSI=2.D0*DELTA*DEXP(-AP*(XI-X0))
      Y0(I)=AKSI**S*DEXP(-AKSI/2.D0)
    3 CONTINUE
      GO TO 5
    1 DO 4 I=1,N
      XI=A+H*(I-1)
      AKSI=2.D0*DELTA*DEXP(-AP*(XI-X0))
      Y0(I)=-AKSI**S*(1.D0-AKSI/(2.D0*S+1.D0))*DEXP(-AKSI/2.D0)
    4 CONTINUE
    5 N2=N-2
      SY2=Y0(N)**2
      DO 6 I=1,N2
      SY2=SY2+Y0(N-I)**2*2.D0
    6 CONTINUE
      SY2=(SY2+Y0(1)**2)*H/2.D0
      ANORM=DSQRT(1.D0/SY2)
      DO 7 I=1,N
      Y0(I)=ANORM*Y0(I)
    7 CONTINUE
      RETURN
      END
C****END**OF**COMPLEX***************************************************
